function [z_out, x_out, time]= RBPF2algo(N,T,par,u,y);

x_      = -ones(par.n_x);                   % A priori estimate of state mean.
Sx_     = -ones(par.n_x,par.n_x);           % A priori estimate of state covariance.
y_      = -ones(par.n_y);                   % A priori estimate of observation mean.
S       = -ones(par.n_y,par.n_y);           % A priori estimate of observation covariance.
x       = -ones(par.n_x, N);                % A posteriori estimate of state mean.
xj      = -ones(par.n_x, N, par.n_m);       % A posteriori estimate of state mean for each psot. state.
Sx      = -ones(par.n_x,par.n_x,N);         % A posteriori estimate of state covariance.
Sxj     = -ones(par.n_x,par.n_x,N,par.n_m); % A posteriori estimate of state covariance for each psot. state.

w          = -ones(1,N);              % Importance weights.
z          = -ones(par.n_z,N);            % A posteriori estimate of discrete states.
z_out      = -ones(par.n_z,T,N);          % Output: discrete states for every time step.
x_out      = -ones(par.n_x,T,N);          % Output: continous states for every time step.


% ============ Sample priors ===================
z  = sample_prior_z(par,N);
x  = sample_prior_x(par,z,N);
% ==============================================
for i=1:N,
  Sx(:,:,i) = par.S0; 
end;

alpha = 0.001;
beta = 2;
kappa = 0;
tic;                  % Initialize timer for benchmarking
for t = 2:T,
    % ============== [1] Unscented Kalman prediction STEP ==============
    for i=1:N
        TM = computeModeTransitionMatrix(x(:,i));
        for j=1:par.n_m
            if TM(z(i),j) == 0
                T_opt(i,j) = 0;
            else
                [xj(:,i,j),Sxj(:,:,i,j),x_,Sx_,y_,nov,S,K] = ukf(x(:,i),Sx(:,:,i),u(t),par.Sigma_p,'ut_ffun',y(:,t),par.Sigma_o,'ut_hfun',j,z(:,i),par,alpha,beta,kappa);
                l(i,j) = ((det(S))^(-0.5)) * exp(-0.5 * nov'*pinv(S)*nov) + 1e-99;
                T_opt(i,j) = l(i,j) * TM(z(i),j);
            end
        end
    end;
  
    % Compute the importance weights.
    w = sum(T_opt,2);
    w = w./sum(w);

    % ==================== [2] Selection STEP ==========================
    if par.resamplingScheme == 1
        outIndex = residualR(1:N,w);        % Higuchi and Liu.
    elseif par.resamplingScheme == 2
        outIndex = deterministicR(1:N,w);   % Kitagawa.
    else  
        outIndex = xltinomialR(1:N,w);     % Ripley, Gordon, etc.  
    end;

    z = z(:,outIndex);               % t-1   <-   t-1
    xj = xj(:,outIndex,:);           % t     <-   t
    Sxj = Sxj(:,:,outIndex,:);       % t     <-   t
    T_opt(:,:) = T_opt(outIndex,:);       % t     <-   t
   
    % ==================== [3] Sampling STEP ===========================
    for i=1:N,  
        T_opt(i,:) = T_opt(i,:)./sum(T_opt(i,:));
        z(1,i) = length(find(cumsum(T_opt(i,:)')<rand))+1;

        j = z(1,i);
        % Use the already computed values for this sampled state j =====
        x(:,i) = xj(:,i,j);
        Sx(:,:,i) = Sxj(:,:,i,j);
    end;
    z_out(:,t,:) = z;
    x_out(:,t,:) = x(:,:);
end;
time = toc;