function [z_out, x_out, time]= RBPFalgo(N,T,par,u,y);

z_pred  = -ones(par.n_z,N);               % A priori estimate of discrete mode
x_      = -ones(par.n_x, N);        % A priori estimate of state mean.
Sx_     = -ones(par.n_x,par.n_x,N); % A priori estimate of state covariance.
y_      = -ones(par.n_y, N);        % A priori estimate of observation mean.
Sy_     = -ones(par.n_y,par.n_y,N); % A priori estimate of observation covariance.
x       = -ones(par.n_x, N);        % A posteriori estimate of state mean.
Sx      = -ones(par.n_x,par.n_x,N); % A posteriori estimate of state covariance.

w          = -ones(1,N);            % Importance weights.
z          = -ones(par.n_z,N);            % A posteriori estimate of discrete states.
z_out      = -ones(par.n_z,T,N);          % Output: discrete states for every time step.
x_out      = -ones(par.n_x,T,N);          % Output: continous states for every time step.

% ============ Sample priors ===================
z  = sample_prior_z(par,N);
x  = sample_prior_x(par,z,N);
% ==============================================
for i=1:N,
    Sx(:,:,i) = par.S0; %FH, changed on Jan12, 2003. eye(par.n_x,par.n_x); 
end;

tic;
for t=2:T,
                  
    % ============ [1] Sequential Importance Sampling STEP ======================
    z_pred = sample_z(par,z,N,z_pred,x);

    % ============ [2] Unscented Kalman Prediction STEP ==================
    alpha = 0.001;
    beta = 2;
    kappa = 0;
    for i=1:N
        [x(:,i),Sx(:,:,i),x_,Sx_,y_,nov,S,K] = ukf(x(:,i),Sx(:,:,i),u(t),par.Sigma_p,'ut_ffun',y(:,t),par.Sigma_o,'ut_hfun',z_pred(:,i),z(:,i),par,alpha,beta,kappa);
        w(i) = ((det(S))^(-0.5)) * exp(-0.5 * nov'*pinv(S)*nov) + 1e-99;        
    end
    w = w./sum(w);
    
    % ============ [4] Selection STEP ===========================================
    if par.resamplingScheme == 1
        outIndex = residualR(1:N,w');        % Higuchi and Liu.
    elseif par.resamplingScheme == 2
        outIndex = deterministicR(1:N,w');   % Kitagawa.
    else  
        outIndex = multinomialR(1:N,w');     % Ripley, Gordon, etc.  
    end;
    z = z_pred(:,outIndex);
    x = x(:,outIndex);
    Sx = Sx(:,:,outIndex);
    z_out(:,t,:) = z(:,:);
    x_out(:,t,:) = x(:,:);            
    
end;
time = toc;