function [z_out, x_out, time] = PFalgo(N,T,par,u,y);

z_pred = -ones(par.n_z, N);   % A priori discrete state estimate.     
x_pred = -ones(par.n_x, N);   % A priori continuous state estimate.
z      = z_pred;              % A posteriori discrete state estimate.     
x      = x_pred;              % A posteriori continuous state estimate.

y_pred = -ones(par.n_y, N);   % One-step-ahead predicted value of y.
w = ones(T,N);                % Importance weights.

% ============ Sample priors ===================
z = sample_prior_z(par,N);
x = sample_prior_x(par,z,N);
% ==============================================

tic;
for t=2:T,

% ============ [1] Sequential Importance Sampling STEP ======================
  z_pred = sample_z(par,z,N,z_pred,x);
  x_pred = sample_x(par,z_pred,z,x,u,t,N,x_pred);

% ============ [2] Reweighting with observation likelihood ==================
  for i=1:N
      y_pred(:,i) = hfun(z_pred(:,i),x_pred(:,i),par,u(:,t));
      Cov(:,:,i) = par.Sigma_o;
  end
  
  for i=1:N
    w(t,i) =  (det(Cov(:,:,i))^(-0.5))*exp(-0.5*(y(:,t)-y_pred(:,i))'* ...
    		    pinv(Cov(:,:,i))*(y(:,t)-y_pred(:,i))) + 1e-99;
  end;  
  w(t,:) = w(t,:)./sum(w(t,:));              % Normalise the weights.

% ============ [3] Selection STEP ===========================================
  if par.resamplingScheme == 1
    outIndex = residualR(1:N,w(t,:)');        % Higuchi and Liu.
  elseif par.resamplingScheme == 2
    outIndex = deterministicR(1:N,w(t,:)');   % Kitagawa.
  else  
    outIndex = multinomialR(1:N,w(t,:)');     % Ripley, Gordon, etc.  
  end;
  z(:,:) = z_pred(:,outIndex);
  x(:,:) = x_pred(:,outIndex);
  z_out(:,t,:) = z(:,:);
  x_out(:,t,:) = x(:,:);
  
end;
time = toc;