function [z_out, x_out, time]= upf(N,T,par,u,y);

z_pred  = -ones(par.n_z,N);         % A priori estimate of discrete mode
x_      = -ones(par.n_x, N);        % A priori estimate of state mean.
Sx_     = -ones(par.n_x,par.n_x,N); % A priori estimate of state covariance.
y_      = -ones(par.n_y, N);        % A priori estimate of observation mean.
Sy_     = -ones(par.n_y,par.n_y,N); % A priori estimate of observation covariance.
x       = -ones(par.n_x, N);        % A posteriori estimate of state mean.
Sx      = -ones(par.n_x,par.n_x,N); % A posteriori estimate of state covariance.

w          = -ones(1,N);            % Importance weights.
z          = -ones(par.n_z,N);            % A posteriori estimate of discrete states.
z_out      = -ones(par.n_z,T,N);          % Output: discrete states for every time step.
x_out      = -ones(par.n_x,T,N);          % Output: continous states for every time step.

x_sampled = -ones(par.n_x, N);      % Sampled continuous parameters for each particle.

% ============ Sample priors ===================
z  = sample_prior_z(par,N);
x  = sample_prior_x(par,z,N);
% ==============================================
for i=1:N,
    Sx(:,:,i) = par.S0; %FH, changed on Jan12, 2003. eye(par.n_x,par.n_x); 
end;

tic;
for t=2:T,
                  
% ============ [1] Sequential Importance Sampling STEP ======================
    z_pred = sample_z(par,z,N,z_pred,x);

% ============ [2] Unscented Kalman Prediction STEP ==================
    % Unscented Transformation through state equation and observation function
    alpha = 0.001;
    beta = 2;
    kappa = 0;
    for i=1:N
        [x(:,i),Sx(:,:,i),x_(:,i),Sx_(:,:,i),y_(:,i),nov,Sy_(:,:,i),K] = ukf(x(:,i),Sx(:,:,i),u(t),par.Sigma_p,'ut_ffun',y(:,t),par.Sigma_o,'ut_hfun',z_pred(:,i),z(:,i),par,alpha,beta,kappa);
    end

    % ============ [4] Sequential Importance Sampling STEP - continuous variables =============
    for i=1:N
%sigma_of_sampling = sqrtm(Sx(:,:,i));
        x_sampled(:,i) = x(:,i) + sqrtm(Sx(:,:,i))*randn(par.n_x,1);
        y_of_x_sampled = hfun(z_pred(:,i),x_sampled(:,i),par,u(:,t));
        % I think this should actually be another unscented transformation and the resulting mean.
        % One could compute a Sy_ for this special sampled value at the same time.

%        lik = (det(par.Sigma_o)^(-0.5))*exp(-0.5*(y(:,t)-y_of_x_sampled)'* ...
%    		    pinv(par.Sigma_o)*(y(:,t)-y_of_x_sampled)) + 1e-99;
        lik = (det(Sy_(:,:,i))^(-0.5))*exp(-0.5*(y(:,t)-y_of_x_sampled)'* ...
    		    pinv(Sy_(:,:,i))*(y(:,t)-y_of_x_sampled)) + 1e-99;

%        prior = (det(par.Sigma_p)^(-0.5))*exp(-0.5*(x_sampled(:,i)-x_(:,i))'* ...
%    		    pinv(par.Sigma_p)*(x_sampled(:,i)-x_(:,i))) + 1e-99;

        prior = (det(Sx_(:,:,i))^(-0.5))*exp(-0.5*(x_sampled(:,i)-x_(:,i))'* ...
    		    pinv(Sx_(:,:,i))*(x_sampled(:,i)-x_(:,i))) + 1e-99;

        proposal = (det(Sx(:,:,i))^(-0.5))*exp(-0.5*(x_sampled(:,i)-x(:,i))'* ...
    		    pinv(Sx(:,:,i))*(x_sampled(:,i)-x(:,i))) + 1e-99;

        w(i) = lik * (prior / proposal);
    end
    w = w./sum(w);
  
    % ============ [5] Selection STEP ===========================================
    if par.resamplingScheme == 1
        outIndex = residualR(1:N,w');        % Higuchi and Liu.
    elseif par.resamplingScheme == 2
        outIndex = deterministicR(1:N,w');   % Kitagawa.
    else  
        outIndex = multinomialR(1:N,w');     % Ripley, Gordon, etc.  
    end;
    z = z_pred(:,outIndex);
    x = x_sampled(:,outIndex); %x(:,outIndex);
    Sx = Sx(:,:,outIndex);
    z_out(:,t,:) = z(:,:);
    x_out(:,t,:) = x(:,:);            
end;
time = toc;