function par = initParameters()
% =======================================================================
%              INITIALISATION AND PARAMETERS
% =======================================================================
% Here, we give you the choice to try three different types of
% resampling algorithms: multinomial (select 3), residual (1) and 
% deterministic (2). Note that the code for these O(N) algorithms is generic.
par.resamplingScheme = 2;    

par.n_x = 6;                   % Number of continuous variables.
par.n_y = 2;                   % Number of observed variables.
par.n_z = 1;                   % Number of discrete states.
par.n_m = 6;                   % Nubmer of discrete modes.

variances = 0.01*[0.001; 0.0001; 0.0005; 0.0001; 0.1; 0.1]; % LB, dLB, D, dD, B, H
par.Sigma_p = diag(variances); % Process noise covariance. 0.00000001 * 
par.Sigma_o = 0.000001*[0.2 0; 0 0.1];  % Observation noise covariance. 0.000001 * 
par.mu0 = [0;0;0;0;8;0];       % Initial Gaussian mean.
par.S0  = par.Sigma_p;         % Initial Gaussian covariance.  

n_x = par.n_x;
n_y = par.n_y;
n_z = par.n_z;
n_m = par.n_m;

% Transition Matrix 
%1->2->5->3->6->4->1
if n_m == 6
    par.T =   [ 0.9     0.1     0       0       0       0
                0.01    0.89    0       0       0.1     0
                0.01    0       0.89    0       0       0.1
                0.1     0       0       0.9     0       0
                0       0       0.2     0       0.8     0
                0       0       0       0.2     0       0.8];

    
    
    % % 	par.T =   [ 0.9     0.1    0       0       0       0
% %                 0.01    0.89    0.05    0       0.05    0
% %                 0.01    0       0.89    0.05    0       0.05
% %                 0.1     0       0       0.9     0       0
% %                 0       0       0.2     0       0.8     0
% %                 0       0       0       0.2     0       0.8];

    par.pz0 = [1 0 0 0 0 0];    % Initial discrete distribution.  
elseif n_m == 1
    par.T = [1];
    par.pz0 = [1];
else
    disp('The transition matrix has an incorrect dimension - not initializing!');
end